@@ -1,4 +1,314 @@
-################################################################################
+## non-exported attribute stuff -------------------------------------------------
+attributesReassign <- function(atts, obj) {
+  attsNames <- setdiff(names(atts), knownAtts)
+  if (length(attsNames))
+    for (att in attsNames) {
+      if (is.null(attr(obj, att))) {
+        attr(obj, att) <- atts[[att]]
+      }
+    }
+  obj
+}
+
+knownAtts <- c("cpp", "class", "attributes", "values", "definition", "pnt")
+
+## non-exported tag stuff -------------------------------------------------------
+
+parseTags <- function(tags) {
+  out <- strsplit(tags, ":")
+  tags2 <- lapply(out, function(x) x[[1]])
+  tags1 <- as.data.table(do.call(rbind, tags2))
+  vals <- vapply(seq_len(NROW(tags1)), function(i) {
+    gsub(paste0(tags1[[1]][i], ":"), "", tags[i])
+  }, FUN.VALUE = character(1))
+  set(tags1, NULL, "tagValue", vals)
+  setnames(tags1, "V1", "tagKey")
+  tags1
+}
+
+tagFilesToLoad <- "filesToLoad"
+tagRelToWhere <- "relToWhere"
+tagOrigFilename <- "origFilename"
+tagOrigRelName <- "origRelName"
+
+tagsSpatRaster <- function(obj = NULL, relToWhere = NULL, relName = NULL, cls = NULL,
+                           whLayers = NULL, layerNams = NULL, obj2 = NULL) {
+  fe <- if (is.null(obj)) NULL else tools::file_ext(obj)
+  c(
+    attr(obj, "tags"),
+    paste0(tagRelToWhere, ":", names(relToWhere)),
+    paste0(tagOrigFilename, ":", basename2(obj)),
+    paste0(tagOrigRelName, ":", relName),
+    paste0("fromDisk:", TRUE),
+    paste0("class:", cls),
+    paste0("fileFormat:", fe),
+    paste0("saveRawFile:", TRUE),
+    paste0("loadFun:", "terra::rast"),
+    paste0("whLayers:", whLayers),
+    paste0("layerNames:", layerNams),
+    paste0(tagFilesToLoad, ":", basename2(obj2))
+  )
+}
+
+## non-exported path functions --------------------------------------------------
+
+absoluteBase <- function(relToWhere, cachePath, ...) {
+  if (identical(relToWhere, "cachePath") && !is.null(cachePath)) {
+    ab <- cachePath
+  } else if (identical(relToWhere, "getwd")) {
+    ab <- getwd()
+  } else {
+    possRelPaths <- modifyListPaths(cachePath, ...)
+    if (relToWhere %in% names(possRelPaths)) {
+      ab <- try(possRelPaths[[relToWhere]])
+    } else {
+      ab <- try(possRelPaths[[1]])
+    }
+    if (is(ab, "try-error")) browser()
+  }
+
+  ab
+}
+
+#' @importFrom utils modifyList
+modifyListPaths <- function(cachePath, ...) {
+  possRelPaths <- list()
+  if (!missing(cachePath))
+    possRelPaths$cachePath <- cachePath
+  dots <- list(...)
+  if (length(dots)) {
+    if (is(dots[[1]], "list")) {
+      if (is.null(names(dots[[1]])))
+        stop("wrapSpatRaster and unwrapSpatRaster require named list passed to dots")
+      possRelPaths <- modifyList(dots[[1]], possRelPaths)
+    }
+  }
+  possRelPaths <- append(possRelPaths, list(getwd = getwd()))
+}
+
+relativeToWhat <- function(file, cachePath, ...) {
+  possRelPaths <- modifyListPaths(cachePath, ...)
+
+  foundAbs <- FALSE
+  dirnameFile <- dirname(file)
+  whSame <- rep(FALSE, length(file))
+  pc <- rep("", length(file))
+  for (nams in names(possRelPaths)) {
+    pc[!whSame] <- mapply(fn = file[!whSame], function(fn)
+      fs::path_common(c(dirname(fn), possRelPaths[[nams]]))
+    )
+
+    out <- sapply(possRelPaths[[nams]], fs::path_rel, path = pc) |> as.character()
+    whSame <- pc == dirnameFile
+    if (all(whSame)) {
+      out <- list(out)
+      names(out) <- nams
+      foundAbs <- TRUE
+      break
+    }
+  }
+  if (isFALSE(foundAbs)) {
+    for (nams in names(possRelPaths)) {
+      out <- dirname(file)
+      names(out) <- ""
+      if (FALSE) { # this is for rebuilding relative against
+        # poss <- fs::path_common(c(file, possRelPaths[nams]))
+        # if (!identical(poss, possRelPaths[nams])) {
+        #   fileRel <- makeRelative(file, poss)
+        #   rel <- makeRelative(possRelPaths[nams], poss)
+        #   relWithDots <- rep("..", length(strsplit(rel, "/|\\\\")[[1]]))
+        #   poss <- file.path(paste(relWithDots, collapse = "/"), dirname(fileRel))
+        #   out <- poss
+        #   out <- list(out)
+        #   names(out) <- nams
+        #   foundAbs <- TRUE
+        #   break
+        # }
+        # names(out) <- nams
+      }
+    }
+  }
+
+  out
+}
+
+## non-exported wrap functions --------------------------------------------------
+
+wrapSpatRaster <- function(obj, cachePath, ...) {
+  fns <- Filenames(obj, allowMultiple = FALSE)
+
+  cls <- class(obj)
+  fnsMulti <- Filenames(obj, allowMultiple = TRUE)
+  obj2 <- asPath(Filenames(obj, allowMultiple = FALSE))
+  nlyrsInFile <- as.integer(terra::nlyr(terra::rast(fns)))
+  layerNams <- paste(names(obj), collapse = layerNamesDelimiter)
+
+  ## A file-backed raster can:
+  ## 1) not be using all the layers in the file; and
+  ## 2) have layer names renamed.
+  whLayers <- seq_along(names(obj))
+  if (!identical(nlyrsInFile, length(names(obj)))) {
+    rr <- terra::rast(fns);
+    objDigs <- unlist(lapply(layerNams, function(ln) .robustDigest(obj[[ln]][])))
+    digs <- character()
+    whLayers <- integer()
+
+    # don't need to go through all layers if the current file has only some; run through from start
+    for (ln in seq_len(terra::nlyr(rr))) {
+      digs[ln] <- .robustDigest(rr[[ln]][])
+      if (digs[ln] %in% objDigs)
+        whLayers <- c(ln, whLayers)
+      if (all(digs %in% objDigs))
+        break
+    }
+  }
+  obj <- asPath(fnsMulti)
+
+  relToWhere <- relativeToWhat(obj, cachePath, ...)
+  # if ("" %in% names(relToWhere)) {
+  #   # absBase <- browser()
+  # }
+  # absBase <- absoluteBase(names(relToWhere), cachePath, ...)
+  relPath <- unname(unlist(relToWhere))
+  relName <- file.path(relPath, basename2(obj))
+
+  tags <- tagsSpatRaster(obj, relToWhere, relName, cls, whLayers, layerNams, obj2)
+  attr(obj, "tags") <- tags
+
+  # c(
+  #   attr(obj, "tags"),
+  #   paste0(tagRelToWhere, ":", names(relToWhere)),
+  #   paste0(tagOrigFilename, ":", basename2(obj)),
+  #   paste0(tagOrigRelName, ":", relName),
+  #   # paste0("origDirname:", dirname(obj)),
+  #   paste0("fromDisk:", TRUE),
+  #   paste0("class:", cls),
+  #   paste0("fileFormat:", tools::file_ext(obj)),
+  #   paste0("saveRawFile:", TRUE),
+  #   paste0("loadFun:", "terra::rast"),
+  #   paste0("whLayers:", whLayers),
+  #   paste0("layerNames:", layerNams),
+  #   paste0(tagFilesToLoad, ":", basename2(obj2))
+  # )
+  obj
+}
+
+unwrapSpatRaster <- function(obj, cachePath, ...) {
+  fns <- Filenames(obj)
+  if (isTRUE(any(nchar(fns) > 0))) {
+    tags <- attr(obj, "tags")
+    if (!is.null(tags)) {
+      if (!is.null(cachePath)) {
+        filenameInCache <- CacheStoredFile(cachePath,
+                                           # cacheId = tools::file_path_sans_ext(basename(obj)),
+                                           obj = obj
+        )
+        feObjs <- file.exists(obj)
+        if (any(feObjs))
+          unlink(obj[feObjs])
+        # fnToLoad <- fns
+        newFiles <- remapFilenames(fns, tags, cachePath, ...)
+        fromFiles <- unlist(filenameInCache)
+      } else {
+        newFiles <- remapFilenames(tags = tags, cachePath = cachePath, ...)
+        fromFiles <- unlist(fns) # fnToLoad <- newFiles$newName
+      }
+      hardLinkOrCopy(fromFiles, newFiles$newName, verbose = 0)
+      # tags <- parseTags(tags)
+      # origRelName <- extractFromCache(tags, tagOrigRelName)
+      # origFilename <- extractFromCache(tags, tagOrigFilename) # tv[tk == tagOrigFilename]
+      # relToWhere <- extractFromCache(tags, "relToWhere")
+      # # possPaths <- modifyListPaths(cachePath, ...)
+      # absBase <- absoluteBase(relToWhere, cachePath, ...)
+      # newName <- file.path(absBase, origRelName)
+
+      # if (FALSE) {
+      #   isAbs <- isAbsolutePath(origRelName)
+      #   if (any(isAbs) || is.null(cachePath)) { # means that it had a specific path, not just relative
+      #     newName2 <- file.path(normPath(extractFromCache(tags, "origDirname")), origFilename)
+      #   } else {
+      #     newName2 <- file.path(cachePath, origRelName)
+      #   }
+      # }
+
+      # if (!identical(newName, newName2)) browser()
+
+      # whFiles <- newFiles$newName[match(basename(extractFromCache(tags, tagFilesToLoad)), origFilename)]
+
+      # if (!is.null(cachePath)) {
+      #   filenameInCache <- CacheStoredFile(cachePath,
+      #                                      # cacheId = tools::file_path_sans_ext(basename(obj)),
+      #                                      obj = obj
+      #   )
+      #   feObjs <- file.exists(obj)
+      #   if (any(feObjs))
+      #     unlink(obj[feObjs])
+      #   fnToLoad <- fns
+      #   hardLinkOrCopy(unlist(filenameInCache), fns, verbose = 0)
+      # } else {
+      #   fnToLoad <- newFiles$newName
+      #   hardLinkOrCopy(unlist(fns), newFiles$newName, verbose = 0)
+      # }
+
+      obj <- eval(parse(text = extractFromCache(newFiles$tagsParsed, "loadFun")))(newFiles$whFiles)
+      possNames <- strsplit(extractFromCache(newFiles$tagsParsed, "layerNames"), split = layerNamesDelimiter)[[1]]
+      namsObjs <- names(obj)
+      if (!identical(possNames, namsObjs)) {
+        whLayers <- as.integer(extractFromCache(newFiles$tagsParsed, "whLayers"))
+        if (length(whLayers) != length(namsObjs)) {
+          obj <- obj[[whLayers]]
+        }
+      }
+
+      # names can be wrong e.g., with "nextNumericName" ... habitatQuality_1 instead of habitatQuality.
+      #  Should use the one without the `nextNumericName`
+      names(obj) <- possNames
+    }
+  }
+  obj
+}
+
+unwrapRaster <- function(obj, cachePath, cacheId) {
+  origFilenames <- if (is(obj, "Raster")) {
+    Filenames(obj) # This is legacy piece which allows backwards compatible
+  } else {
+    obj$origRaster
+  }
+
+  filesExist <- file.exists(origFilenames)
+  cacheFilenames <- Filenames(obj)
+  filesExistInCache <- file.exists(cacheFilenames)
+  if (any(!filesExistInCache)) {
+    fileTails <- gsub("^.+(rasters.+)$", "\\1", cacheFilenames)
+    correctFilenames <- file.path(cachePath, fileTails)
+    filesExistInCache <- file.exists(correctFilenames)
+    if (all(filesExistInCache)) {
+      cacheFilenames <- correctFilenames
+    } else {
+      stop("File-backed raster files in the cache are corrupt for cacheId: ", cacheId)
+    }
+  }
+  out <- hardLinkOrCopy(cacheFilenames[filesExistInCache],
+                        origFilenames[filesExistInCache],
+                        overwrite = TRUE
+  )
+
+  newOutput <- updateFilenameSlots(obj$cacheRaster,
+                                   Filenames(obj, allowMultiple = FALSE),
+                                   newFilenames = grep("\\.gri$", origFilenames, value = TRUE, invert = TRUE)
+  )
+  obj <- newOutput
+  obj <- .setSubAttrInList(obj, ".Cache", "newCache", FALSE)
+  obj
+}
+
+## misc non-exported stuff ------------------------------------------------------
+grepStartsTwoDots <- "^\\.\\."
+
+## exported generics and functions ----------------------------------------------
+
+### .tagsByClass ----------------------------------------------------------------
+
 #' Exported generics and methods
 #'
 #' There are a number of generics that are exported for other packages to use.
@@ -39,7 +349,8 @@ setMethod(
   }
 )
 
-################################################################################
+### .cacheMessage ---------------------------------------------------------------
+
 #' @details
 #' `.cacheMessage` should make a call to `message` that gives information about
 #' the loaded cached object being returned.
@@ -73,24 +384,51 @@ setMethod(
   ".cacheMessage",
   signature = "ANY",
   definition = function(object, functionName, fromMemoise, verbose = getOption("reproducible.verbose", 1)) {
+    postMess <- NULL
+    whMessage <- .message$LoadedCacheResult()
     if (isTRUE(fromMemoise)) {
-      whMessage <- .loadedMemoisedResultMsg
-      messageCache(.loadedCacheMsg(whMessage, functionName), verbose = verbose)
-    } else if (!is.na(fromMemoise)) {
-      whMessage <- .loadedCacheResultMsg
-      messageCache(.loadedCacheMsg(whMessage, functionName), " ",
-                   .addingToMemoisedMsg,
-                   sep = "", verbose = verbose
-      )
-    } else {
-      whMessage <- .loadedCacheResultMsg
-      messageCache(.loadedCacheMsg(whMessage, functionName), verbose = verbose)
+      whMessage <- .message$LoadedCacheResult(2)
+    } else if (fromMemoise %in% FALSE) {
+      postMess <- paste0(" ", .message$AddingToMemoised)
     }
+    baseMess <- .message$LoadedCache(whMessage, functionName)
+    if (!is.null(postMess))
+      baseMess <- paste0(baseMess, postMess)
+    messageCache(baseMess, verbose = verbose)
     return(invisible(whMessage))
   }
 )
 
-################################################################################
+### .cacheMessageObjectToRetrieve -------------------------------------------------------------
+
+#' @param fullCacheTableForObj The data.table entry from the Cache database for only
+#' this `cacheId`, e.g., via `showCache()`.
+#' @inheritParams Cache
+#' @inheritParams .unwrap
+#'
+#' @details
+#' `.objecxtToRetrieveMessage` is the messaging for recovering an object from Cache.
+#'
+#' @export
+#' @rdname exportedMethods
+.cacheMessageObjectToRetrieve <- function(functionName, fullCacheTableForObj, cachePath, cacheId, verbose) {
+  objSize <- as.numeric(tail(extractFromCache(fullCacheTableForObj, elem = "file.size"), 1))
+  class(objSize) <- "object_size"
+  bigFile <- isTRUE(objSize > 1e6)
+
+  fileFormat <- unique(extractFromCache(fullCacheTableForObj, elem = "fileFormat")) # can have a single tif for many entries
+
+  messageCache(.message$ObjToRetrieveFn(functionName), ", ",
+  #             messageCache("...(Object to retrieve (fn: ", .messageFunctionFn(functionName), ", ",
+               basename2(CacheStoredFile(cachePath, cacheId, format = fileFormat)),
+               ")",
+               if (bigFile) " is large: ",
+               if (bigFile) format(objSize, units = "auto"),
+               " ... ",
+               verbose = verbose
+  )
+}
+
 #' @details
 #' `.addTagsToOutput` should add one or more attributes to an object, named either
 #' `"tags"`, `"call"` or `"function"`. It may be wise to do a "deep" copy within this
@@ -124,7 +462,9 @@ setMethod(
   }
 )
 
-################################################################################
+
+### .preDigestByClass ---------------------------------------------------------------------------
+
 #' `preDigestByClass`: A function called to do any miscellaneous things to
 #' do before `.robustDigest` and after `FUN` call
 #'
@@ -156,7 +496,8 @@ setMethod(
   }
 )
 
-################################################################################
+### .checkCacheRepo ---------------------------------------------------------------------------
+
 #' `.checkCacheRepo`: normally, `checkPath` can be called directly, and if more
 #' friendly, but that does not have class-specific methods.
 #'
@@ -180,6 +521,7 @@ setGeneric(".checkCacheRepo", function(object, create = FALSE,
 
 #' @export
 #' @rdname exportedMethods
+#' @include messages.R
 setMethod(
   ".checkCacheRepo",
   signature = "ANY",
@@ -191,14 +533,14 @@ setMethod(
         #  If no, then user is aware and doesn't need a message
         if (any(grepl(normPath(tmpDir), normPath(getOption("reproducible.cachePath")))) ||
             any(grepl(normPath(tempdir()), normPath(getOption("reproducible.cachePath"))))) {
-          messageCache("No cachePath supplied and getOption('reproducible.cachePath') is inside a temporary directory;\n",
+          messageCache(.message$NoCachePathSupplied, " and getOption('reproducible.cachePath') is inside a temporary directory;\n",
                        "  this will not persist across R sessions.",
                        verbose = verbose
           )
         }
         getOption("reproducible.cachePath", tmpDir)
       } else {
-        messageCache("No cachePath supplied. Using ", .reproducibleTempCacheDir(), verbose = verbose)
+        messageCache(.message$NoCachePathSupplied, ". Using ", .reproducibleTempCacheDir(), verbose = verbose)
         .reproducibleTempCacheDir()
       }
       checkPath(path = cachePath, create = create)
@@ -206,7 +548,8 @@ setMethod(
   }
 )
 
-################################################################################
+### .prepareOutput ----------------------------------------------------------------------------
+
 #'  @details
 #'  `.prepareOutput` allows a package developer to do whatever is needed to prepare
 #'  the output on recovery from the cache repository. In other words,
@@ -235,7 +578,7 @@ setMethod(
 #'   r <- terra::rast(terra::ext(0, 10, 0, 10), vals = 1:100)
 #'
 #'   # write to disk manually -- will be in tempdir()
-#'   r <- terra::writeRaster(r, file = tempfile(fileext = ".tif"))
+#'   r <- terra::writeRaster(r, filename = tempfile(fileext = ".tif"))
 #'
 #'   # copy it to the cache repository
 #'   r <- .prepareOutput(r, tempdir())
@@ -244,7 +587,6 @@ setGeneric(".prepareOutput", function(object, cachePath, ...) {
   standardGeneric(".prepareOutput")
 })
 
-
 #' @export
 #' @rdname exportedMethods
 setMethod(
@@ -261,7 +603,8 @@ setMethod(
   }
 )
 
-################################################################################
+### .addChangedAttr ---------------------------------------------------------------------------
+
 #' @details
 #' `.addChangedAttr` should return the same object, with this a very specific
 #' attribute added: it must be named ".Cache", and have a sub-list element named
@@ -301,6 +644,57 @@ setMethod(
   }
 )
 
+#' Remap file names
+#'
+#' Update file path metadata for file-backed objects (e.g., `SpatRasters`).
+#' Useful when moving saved objects between projects or machines.
+#'
+#' @param obj (optional) object whose file path metadata will be remapped
+#' @param tags cache tags `data.table` object
+#' @param cachePath character string specifying the path to the cache directory or `NULL`
+#' @param ... Additional path arguments, passed to `absoluteBase()` and `modifyListPaths()`
+#'
+#' @export
+#' @importFrom fs path_join path_norm
+remapFilenames <- function(obj, tags, cachePath, ...) {
+  tags <- parseTags(tags)
+  origFilename <- extractFromCache(tags, tagOrigFilename) # tv[tk == tagOrigFilename]
+
+  if (missing(obj)) {
+    origRelName <- extractFromCache(tags, tagOrigRelName) |> fs::path_norm()
+    relToWhere <- extractFromCache(tags, "relToWhere")
+
+    ## NOTE: extractFromCache() is looking for specific tags which may not exist if saved
+    ## using earlier versions of the package, and cannot be restored.
+    if (is.null(relToWhere) || length(relToWhere) == 0) {
+      stop("remapFileNames() cannot restore objects saved using a previous version of 'reproducible'.")
+    }
+
+    possRelPaths <- modifyListPaths(cachePath, ...)
+    if (relToWhere %in% names(possRelPaths)) {
+      absBase <- absoluteBase(relToWhere, cachePath, ...)
+    } else {
+      absBase <- possRelPaths[[1]]
+      isOutside <- grepl(grepStartsTwoDots, origRelName)
+      if (any(isOutside)) {
+        ## means the relative path is "outside" of something;
+        ## strip all ".." if relToWhere doesn't exist
+        while (any(grepl(grepStartsTwoDots, origRelName))) {
+          origRelName <- gsub(paste0(grepStartsTwoDots, "|(\\\\|/)"), "", origRelName)
+        }
+      }
+    }
+    newName <- vapply(origRelName, function(x) {
+      fs::path_join(c(absBase, x)) |> fs::path_norm()
+    }, character(1))
+  } else {
+    newName <- obj
+  }
+
+  whFiles <- newName[match(basename(extractFromCache(tags, tagFilesToLoad)), origFilename)]
+  list(newName = newName, whFiles = whFiles, tagsParsed = tags)
+}
+
 #' @details
 #' `updateFilenameSlots`: this exists because when copying file-backed rasters, the
 #' usual mechanism of `writeRaster` can be very slow. This function allows
@@ -487,15 +881,14 @@ unmakeMemoisable.default <- function(x) {
 #' @param ... Arguments passed to methods; default does not use anything in `...`.
 #' @inheritParams Cache
 #' @inheritParams loadFromCache
-#' @rdname dotWrap
-#' @return
-#' Returns an object that can be saved to disk e.g., via `saveRDS`.
+#'
+#' @return Returns an object that can be saved to disk e.g., via `saveRDS`.
 #'
 #' @export
-#'
+#' @rdname dotWrap
 .wrap <- function(obj, cachePath, preDigest,  drv = getDrv(getOption("reproducible.drv", NULL)),
                   conn = getOption("reproducible.conn", NULL),
-                  verbose = getOption("reproducible.verbose"), ...) {
+                  verbose = getOption("reproducible.verbose"), outputObjects  = NULL, ...) {
   UseMethod(".wrap")
 }
 
@@ -503,7 +896,14 @@ unmakeMemoisable.default <- function(x) {
 #' @rdname dotWrap
 .wrap.list <- function(obj, cachePath, preDigest, drv = getDrv(getOption("reproducible.drv", NULL)),
                        conn = getOption("reproducible.conn", NULL),
-                       verbose = getOption("reproducible.verbose"), ...) {
+                       verbose = getOption("reproducible.verbose"), outputObjects = NULL, ...) {
+
+  if (!is.null(outputObjects)) {
+    allObjs <- names(obj)
+    nullify <- setdiff(allObjs, outputObjects)
+    obj[nullify] <- NULL
+  }
+
   attrsOrig <- attributes(obj)
   obj <- lapply(obj, .wrap, preDigest = preDigest, cachePath = cachePath, drv = drv,
                 conn = conn, verbose = verbose, ...)
@@ -518,7 +918,7 @@ unmakeMemoisable.default <- function(x) {
     attrsOrig["tags"] <- newList
   }
   if (!is.null(attrsOrig)) {
-    for (tt in c(".Cache", "tags", "call"))
+    for (tt in intersect(names(attrsOrig), c(".Cache", "tags", "call")))
       attr(obj, tt) <- attrsOrig[[tt]]
   }
   obj
@@ -528,14 +928,23 @@ unmakeMemoisable.default <- function(x) {
 #' @rdname dotWrap
 .wrap.environment <- function(obj, cachePath, preDigest, drv = getDrv(getOption("reproducible.drv", NULL)),
                               conn = getOption("reproducible.conn", NULL),
-                              verbose = getOption("reproducible.verbose"), ...) {
-  obj2 <- as.list(obj, all.names = FALSE)
-  out <- .wrap(obj2, cachePath = cachePath, preDigest = preDigest, drv = drv,
-               conn = conn, verbose = verbose, ...)
-  obj <- Copy(obj)
-  obj2 <- list2envAttempts(out, obj)
-  if (!is.null(obj2)) obj <- obj2
+                              verbose = getOption("reproducible.verbose"), outputObjects = NULL, ...) {
 
+  if (!is.null(outputObjects)) {
+    allObjs <- ls(obj)
+    nullify <- setdiff(allObjs, outputObjects)
+    rm(list = nullify, envir = obj)
+  }
+
+  if (length(ls(obj, all.names = T)) > 0) {
+  obj2 <- as.list(obj, all.names = TRUE)
+  out <- .wrap(obj2, cachePath = cachePath, preDigest = preDigest, drv = drv,
+               conn = conn, verbose = verbose, outputObjects = outputObjects, ...)
+    # obj <- Copy(obj)
+    obj2 <- list2envAttempts(out, obj)
+    if (!is.null(obj2)) obj <- obj2
+
+  }
   obj
 }
 
@@ -548,17 +957,17 @@ unmakeMemoisable.default <- function(x) {
 #'   exWrapped <- .wrap(ex)
 #'   ex1 <- .unwrap(exWrapped)
 #' }
-
 #'
 .wrap.default <- function(obj, cachePath, preDigest, drv = getDrv(getOption("reproducible.drv", NULL)),
                           conn = getOption("reproducible.conn", NULL),
                           verbose = getOption("reproducible.verbose"), ...) {
   rasters <- is(obj, "Raster")
+  atts <- attributes(obj)
+  reassignAtts <- TRUE
 
   if (any(rasters)) {
     .requireNamespace("raster", stopOnFALSE = TRUE)
     objOrig <- obj
-    atts <- attributes(obj)
     obj <- .prepareFileBackedRaster(obj,
                                     repoDir = cachePath,
                                     overwrite = FALSE, drv = drv, conn = conn
@@ -604,12 +1013,17 @@ unmakeMemoisable.default <- function(x) {
     }
   }
 
-  if (any(inherits(obj, c("SpatVector", "SpatRaster", "SpatExtent")))) {
+  if (is(obj, "SpatVectorCollection")) {
+    obj <- .wrap(as.list(obj))
+    class(obj) <- "PackedSpatVectorCollection"
+  }
+
+  if (any(inherits(obj, c("SpatVector", "SpatRaster", "SpatExtent", "data.table")))) {
     if (!requireNamespace("terra", quietly = TRUE)) {
       stop("Please install terra package")
     }
-    messageCache("...wrapping terra object for saving...", verboseLevel = 2, verbose = verbose)
-    attrs <- attr(obj, ".Cache")
+    messageCache("wrapping terra object for saving...", verboseLevel = 2, verbose = verbose)
+    # attrs <- attr(obj, ".Cache")
 
     # next is for terra objects --> terra::wrap is ridiculously slow for SpatVector objects; use
     #   custom version in reproducible where here
@@ -629,16 +1043,26 @@ unmakeMemoisable.default <- function(x) {
       )
       attr(obj, "class") <- "PackedSpatExtent"
       useWrap <- FALSE
+      reassignAtts <- FALSE
+    }
+    if (is(obj, "data.table")) {
+      obj <- data.table::copy(obj)
+      useWrap <- FALSE
     }
 
     if (useWrap) {
       obj <- terra::wrap(obj)
     } # let method dispatch work
 
-    attr(obj, ".Cache") <- attrs
+    # attr(obj, ".Cache") <- attrs
 
     messageCache("\b Done!", verboseLevel = 2, verbose = verbose)
   }
+
+  # put attributes back on the potentially packed object
+  if (isTRUE(reassignAtts))
+    obj <- attributesReassign(atts, obj)
+
   obj
 }
 
@@ -647,6 +1071,11 @@ unmakeMemoisable.default <- function(x) {
 .unwrap.default <- function(obj, cachePath, cacheId,
                             drv = getDrv(getOption("reproducible.drv", NULL)),
                             conn = getOption("reproducible.conn", NULL), ...) {
+  atts <- attributes(obj)
+  if (any(inherits(obj, "PackedSpatVectorCollection"))) {
+    obj <- lapply(obj, .unwrap)
+    obj <- terra::svc(obj)
+  }
   if (any(inherits(obj, c("PackedSpatVector", "PackedSpatRaster", "PackedSpatExtent")))) {
     if (!requireNamespace("terra")) stop("Please install.packages('terra')")
     if (any(inherits(obj, "PackedSpatVector"))) {
@@ -661,147 +1090,12 @@ unmakeMemoisable.default <- function(x) {
   } else if (is(obj, "Path")) {
     obj <- unwrapSpatRaster(obj, cachePath, ...)
   }
+  # put attributes back on the potentially packed object
+  obj <- attributesReassign(atts, obj)
 
   obj
 }
 
-wrapSpatRaster <- function(obj, cachePath, ...) {
-  cls <- class(obj)
-  fns <- Filenames(obj, allowMultiple = FALSE)
-  fnsMulti <- Filenames(obj, allowMultiple = TRUE)
-  obj2 <- asPath(Filenames(obj, allowMultiple = FALSE))
-  nlyrsInFile <- as.integer(terra::nlyr(terra::rast(fns)))
-  layerNams <- paste(names(obj), collapse = layerNamesDelimiter)
-
-  # A file-backed rast can 1) not be using all the layers in the file and
-  # 2) have layer names renamed
-  whLayers <- seq_along(names(obj))
-  if (!identical(nlyrsInFile, length(names(obj)))) {
-    rr <- terra::rast(fns);
-    objDigs <- unlist(lapply(layerNams, function(ln) .robustDigest(obj[[ln]][])))
-    digs <- character()
-    whLayers <- integer()
-
-    # don't need to go through all layers if the current file has only some; run through from start
-    for (ln in seq_len(terra::nlyr(rr))) {
-      digs[ln] <- .robustDigest(rr[[ln]][])
-      if (digs[ln] %in% objDigs)
-        whLayers <- c(ln, whLayers)
-      if (all(digs %in% objDigs))
-        break
-    }
-  }
-  obj <- asPath(fnsMulti)
-
-  relToWhere <- relativeToWhat(obj, cachePath, ...)
-  # if ("" %in% names(relToWhere)) {
-  #   # absBase <- browser()
-  # }
-  # absBase <- absoluteBase(names(relToWhere), cachePath, ...)
-  relPath <- unname(unlist(relToWhere))
-  relName <- file.path(relPath, basename2(obj))
-
-  tags <- tagsSpatRaster(obj, relToWhere, relName, cls, whLayers, layerNams, obj2)
-  attr(obj, "tags") <- tags
-
-  # c(
-  #   attr(obj, "tags"),
-  #   paste0(tagRelToWhere, ":", names(relToWhere)),
-  #   paste0(tagOrigFilename, ":", basename2(obj)),
-  #   paste0(tagOrigRelName, ":", relName),
-  #   # paste0("origDirname:", dirname(obj)),
-  #   paste0("fromDisk:", TRUE),
-  #   paste0("class:", cls),
-  #   paste0("fileFormat:", tools::file_ext(obj)),
-  #   paste0("saveRawFile:", TRUE),
-  #   paste0("loadFun:", "terra::rast"),
-  #   paste0("whLayers:", whLayers),
-  #   paste0("layerNames:", layerNams),
-  #   paste0(tagFilesToLoad, ":", basename2(obj2))
-  # )
-  obj
-}
-
-tagFilesToLoad = "filesToLoad"
-tagRelToWhere = "relToWhere"
-tagOrigFilename <- "origFilename"
-tagOrigRelName <- "origRelName"
-
-unwrapSpatRaster <- function(obj, cachePath, ...) {
-  fns <- Filenames(obj)
-  if (isTRUE(any(nchar(fns) > 0))) {
-    tags <- attr(obj, "tags")
-    if (!is.null(tags)) {
-      if (!is.null(cachePath)) {
-        filenameInCache <- CacheStoredFile(cachePath,
-                                           # cacheId = tools::file_path_sans_ext(basename(obj)),
-                                           obj = obj
-        )
-        feObjs <- file.exists(obj)
-        if (any(feObjs))
-          unlink(obj[feObjs])
-        # fnToLoad <- fns
-        newFiles <- remapFilenames(fns, tags, cachePath, ...)
-        fromFiles <- unlist(filenameInCache)
-      } else {
-        newFiles <- remapFilenames(tags = tags, cachePath = cachePath, ...)
-        fromFiles <- unlist(fns) # fnToLoad <- newFiles$newName
-      }
-      hardLinkOrCopy(fromFiles, newFiles$newName, verbose = 0)
-      # tags <- parseTags(tags)
-      # origRelName <- extractFromCache(tags, tagOrigRelName)
-      # origFilename <- extractFromCache(tags, tagOrigFilename) # tv[tk == tagOrigFilename]
-      # relToWhere <- extractFromCache(tags, "relToWhere")
-      # # possPaths <- modifyListPaths(cachePath, ...)
-      # absBase <- absoluteBase(relToWhere, cachePath, ...)
-      # newName <- file.path(absBase, origRelName)
-
-      # if (FALSE) {
-      #   isAbs <- isAbsolutePath(origRelName)
-      #   if (any(isAbs) || is.null(cachePath)) { # means that it had a specific path, not just relative
-      #     newName2 <- file.path(normPath(extractFromCache(tags, "origDirname")), origFilename)
-      #   } else {
-      #     newName2 <- file.path(cachePath, origRelName)
-      #   }
-      # }
-
-      # if (!identical(newName, newName2)) browser()
-
-      # whFiles <- newFiles$newName[match(basename(extractFromCache(tags, tagFilesToLoad)), origFilename)]
-
-      # if (!is.null(cachePath)) {
-      #   filenameInCache <- CacheStoredFile(cachePath,
-      #                                      # cacheId = tools::file_path_sans_ext(basename(obj)),
-      #                                      obj = obj
-      #   )
-      #   feObjs <- file.exists(obj)
-      #   if (any(feObjs))
-      #     unlink(obj[feObjs])
-      #   fnToLoad <- fns
-      #   hardLinkOrCopy(unlist(filenameInCache), fns, verbose = 0)
-      # } else {
-      #   fnToLoad <- newFiles$newName
-      #   hardLinkOrCopy(unlist(fns), newFiles$newName, verbose = 0)
-      # }
-
-      obj <- eval(parse(text = extractFromCache(newFiles$tagsParsed, "loadFun")))(newFiles$whFiles)
-      possNames <- strsplit(extractFromCache(newFiles$tagsParsed, "layerNames"), split = layerNamesDelimiter)[[1]]
-      namsObjs <- names(obj)
-      if (!identical(possNames, namsObjs)) {
-        whLayers <- as.integer(extractFromCache(newFiles$tagsParsed, "whLayers"))
-        if (length(whLayers) != length(namsObjs)) {
-          obj <- obj[[whLayers]]
-        }
-      }
-
-      # names can be wrong e.g., with "nextNumericName" ... habitatQuality_1 instead of habitatQuality.
-      #  Should use the one without the `nextNumericName`
-      names(obj) <- possNames
-    }
-  }
-  obj
-}
-
 #' @export
 #' @param cacheId Used strictly for messaging. This should be the cacheId of the object being recovered.
 #' @rdname dotWrap
@@ -817,12 +1111,17 @@ unwrapSpatRaster <- function(obj, cachePath, ...) {
                                 drv = getDrv(getOption("reproducible.drv", NULL)),
                                 conn = getOption("reproducible.conn", NULL), ...) {
   # the as.list doesn't get everything. But with a simList, this is OK; rest will stay
-  objList <- as.list(obj) # don't overwrite everything, just the ones in the list part
+  atts <- attributes(obj)
+  # if (!is.null(obj$fireSense_dataPrepFit$.objects$studyAreaUnion)) browser()
+  objList <- as.list(obj, all.names = TRUE) # don't overwrite everything, just the ones in the list part; but need .mods, .objects etc.
 
   outList <- .unwrap(objList, cachePath = cachePath, cacheId = cacheId, drv = drv, conn = conn, ...)
   output2 <- list2envAttempts(outList, obj) # don't return it if the list2env retured nothing (a normal environment situation; not simList)
   if (!is.null(output2)) obj <- output2
 
+  # put attributes back on the potentially packed object
+  obj <- attributesReassign(atts, obj)
+
   obj
 }
 
@@ -831,6 +1130,7 @@ unwrapSpatRaster <- function(obj, cachePath, ...) {
 .unwrap.list <- function(obj, cachePath, cacheId,
                          drv = getDrv(getOption("reproducible.drv", NULL)),
                          conn = getOption("reproducible.conn", NULL), ...) {
+  atts <- attributes(obj)
   anyNames <- names(obj)
   isSpatVector <- if (is.null(anyNames)) FALSE else all(names(obj) %in% spatVectorNamesForCache)
   if (isTRUE(isSpatVector)) {
@@ -845,179 +1145,8 @@ unwrapSpatRaster <- function(obj, cachePath, ...) {
       })
     }
   }
-}
+  # put attributes back on the potentially packed object
+  obj <- attributesReassign(atts, obj)
 
-unwrapRaster <- function(obj, cachePath, cacheId) {
-  origFilenames <- if (is(obj, "Raster")) {
-    Filenames(obj) # This is legacy piece which allows backwards compatible
-  } else {
-    obj$origRaster
-  }
-
-  filesExist <- file.exists(origFilenames)
-  cacheFilenames <- Filenames(obj)
-  filesExistInCache <- file.exists(cacheFilenames)
-  if (any(!filesExistInCache)) {
-    fileTails <- gsub("^.+(rasters.+)$", "\\1", cacheFilenames)
-    correctFilenames <- file.path(cachePath, fileTails)
-    filesExistInCache <- file.exists(correctFilenames)
-    if (all(filesExistInCache)) {
-      cacheFilenames <- correctFilenames
-    } else {
-      stop("File-backed raster files in the cache are corrupt for cacheId: ", cacheId)
-    }
-  }
-  out <- hardLinkOrCopy(cacheFilenames[filesExistInCache],
-                        origFilenames[filesExistInCache],
-                        overwrite = TRUE
-  )
-
-  newOutput <- updateFilenameSlots(obj$cacheRaster,
-                                   Filenames(obj, allowMultiple = FALSE),
-                                   newFilenames = grep("\\.gri$", origFilenames, value = TRUE, invert = TRUE)
-  )
-  obj <- newOutput
-  obj <- .setSubAttrInList(obj, ".Cache", "newCache", FALSE)
   obj
 }
-
-parseTags <- function(tags) {
-  out <- strsplit(tags, ":")
-  tags2 <- lapply(out, function(x) x[[1]])
-  tags1 <- as.data.table(do.call(rbind, tags2))
-  vals <- vapply(seq_len(NROW(tags1)), function(i) {
-    gsub(paste0(tags1[[1]][i], ":"), "", tags[i])
-  }, FUN.VALUE = character(1))
-  set(tags1, NULL, "tagValue", vals)
-  setnames(tags1, "V1", "tagKey")
-  tags1
-}
-
-relativeToWhat <- function(file, cachePath, ...) {
-  possRelPaths <- modifyListPaths(cachePath, ...)
-
-  foundAbs <- FALSE
-  dirnameFile <- dirname(file)
-  whSame <- rep(FALSE, length(file))
-  pc <- rep("", length(file))
-  for (nams in names(possRelPaths)) {
-    pc[!whSame] <- mapply(fn = file[!whSame], function(fn)
-      fs::path_common(c(dirname(fn), possRelPaths[[nams]]))
-    )
-
-    out <- as.character(fs::path_rel(pc, possRelPaths[[nams]]))
-    whSame <- pc == dirnameFile
-    if (all(whSame)) {
-      out <- list(out)
-      names(out) <- nams
-      foundAbs <- TRUE
-      break
-    }
-  }
-  if (isFALSE(foundAbs)) {
-    for (nams in names(possRelPaths)) {
-      out <- dirname(file)
-      names(out) <- ""
-      if (FALSE) { # this is for rebuilding relative against
-      # poss <- fs::path_common(c(file, possRelPaths[nams]))
-      # if (!identical(poss, possRelPaths[nams])) {
-      #   fileRel <- makeRelative(file, poss)
-      #   rel <- makeRelative(possRelPaths[nams], poss)
-      #   relWithDots <- rep("..", length(strsplit(rel, "/|\\\\")[[1]]))
-      #   poss <- file.path(paste(relWithDots, collapse = "/"), dirname(fileRel))
-      #   out <- poss
-      #   out <- list(out)
-      #   names(out) <- nams
-      #   foundAbs <- TRUE
-      #   break
-      # }
-      # names(out) <- nams
-      }
-    }
-  }
-
-  out
-}
-
-absoluteBase <- function(relToWhere, cachePath, ...) {
-  if (identical(relToWhere, "cachePath") && !is.null(cachePath)) {
-    ab <- cachePath
-  } else if (identical(relToWhere, "getwd")) {
-    ab <- getwd()
-  } else {
-    possRelPaths <- modifyListPaths(cachePath, ...)
-    if (relToWhere %in% names(possRelPaths)) {
-      ab <- try(possRelPaths[[relToWhere]])
-    } else {
-      ab <- try(possRelPaths[[1]])
-    }
-    if (is(ab, "try-error")) browser()
-  }
-
-  ab
-}
-
-#' @importFrom utils modifyList
-modifyListPaths <- function(cachePath, ...) {
-  possRelPaths <- list()
-  if (!missing(cachePath))
-    possRelPaths$cachePath <- cachePath
-  dots <- list(...)
-  if (length(dots)) {
-    if (is(dots[[1]], "list")) {
-      if (is.null(names(dots[[1]])))
-        stop("wrapSpatRaster and unwrapSpatRaster require named list passed to dots")
-      possRelPaths <- modifyList(dots[[1]], possRelPaths)
-    }
-  }
-  possRelPaths <- append(possRelPaths, list(getwd = getwd()))
-}
-
-tagsSpatRaster <- function(obj = NULL, relToWhere = NULL, relName = NULL, cls = NULL,
-                           whLayers = NULL, layerNams = NULL, obj2 = NULL) {
-  fe <- if (is.null(obj)) NULL else tools::file_ext(obj)
-  c(
-    attr(obj, "tags"),
-    paste0(tagRelToWhere, ":", names(relToWhere)),
-    paste0(tagOrigFilename, ":", basename2(obj)),
-    paste0(tagOrigRelName, ":", relName),
-    paste0("fromDisk:", TRUE),
-    paste0("class:", cls),
-    paste0("fileFormat:", fe),
-    paste0("saveRawFile:", TRUE),
-    paste0("loadFun:", "terra::rast"),
-    paste0("whLayers:", whLayers),
-    paste0("layerNames:", layerNams),
-    paste0(tagFilesToLoad, ":", basename2(obj2))
-  )
-}
-
-remapFilenames <- function(obj, tags, cachePath, ...) {
-  tags <- parseTags(tags)
-  origFilename <- extractFromCache(tags, tagOrigFilename) # tv[tk == tagOrigFilename]
-  if (missing(obj)) {
-    origRelName <- extractFromCache(tags, tagOrigRelName)
-    relToWhere <- extractFromCache(tags, "relToWhere")
-    possRelPaths <- modifyListPaths(cachePath, ...)
-    if (relToWhere %in% names(possRelPaths)) {
-      absBase <- absoluteBase(relToWhere, cachePath, ...)
-    } else {
-      absBase <- possRelPaths[[1]]
-      isOutside <- grepl(grepStartsTwoDots, origRelName)
-      if (any(isOutside)) {
-        # means the relative path is "outside" of something ... strip all ".." if relToWhere doesn't exist
-        while(any(grepl(grepStartsTwoDots, origRelName))) {
-          origRelName <- gsub(paste0(grepStartsTwoDots, "|(\\\\|/)"), "", origRelName)
-        }
-      }
-    }
-    newName <- file.path(absBase, origRelName)
-  } else {
-    newName <- obj
-  }
-
-  whFiles <- newName[match(basename(extractFromCache(tags, tagFilesToLoad)), origFilename)]
-  list(newName = newName, whFiles = whFiles, tagsParsed = tags)
-}
-
-grepStartsTwoDots <- "^\\.\\."
